/*****************************************************************************************
 *
 *  Copyright 2006+ by Cieszak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 ****************************************************************************************
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        2006.07.19    Cieszak            Inital Release (tfstextcount,
 *                                              tfstextreadln, tfstextappendln)
 *
 *  1.1        2006.08.03    Cieszak            Add rest of functions
 *                                              (tfstextinsertln, tfstextreplaceln,
 *                                              tfstextdeleteln, tfstextmode)
 *
 *  1.2        2006.08.14    Cieszak            Full working version
 *
 *  1.3        2006.08.28    Cieszak            Rewrited and optimized all of the code 
 *	                                             (added tfstextgotonextln, tfstextgotoln,
 *	                                             tfstextreadnextln, tfstextwriteeol,
 *	                                             tfstextwriteln functions)
 *
 *  1.4        2006.09.22    Cieszak            Another optimizations
 *
 *  1.5        2006.12.04    Cieszak            Corrected documentation
 *
 ****************************************************************************************/

#ifndef TFSTEXT_H
#define TFSTEXT_H

#include <nokix/all.h>

//file EOF type
#define WIN_MODE 1
#define LIN_MODE 2
#define MAC_MODE 3

//max length of memory buffer used for reading
#define MAX_MEM_BUFFER 320

//error code - this is not text file
#define TFSERR_NOTTEXTFILE -20

//#define DEBUG

#ifdef DEBUG
#define dprintf printf
#else
#define dprintf(...)
#endif

/*
   Function: tfstextreadnextln
   Read line from current position to eol-char

   Parameters:
      fd - file descriptor
      text - buffer for readed line
      len - length of buffer

   Return:
      number of readed bytes (or error code)
*/
int tfstextreadnextln(int fd, char *text, const int len);

/*
   Function: tfstextreadln
   Read line form file to buffer

   Parameters:
      fd - file descriptor
      num - number of line to read
      text - buffer where read line to
      len - length of buffer

   Return:
      number of readed bytes (or error code)

*/
int tfstextreadln(int fd, const int num, char *text, int len);

/*
   Function: tfstextcount
   Count lines in file

   Parameters:
      fd - file descriptor
      maxlnlen - pointer to variable which will be set to maximal length of line
         in current file; if don't want this information, give NULL as second parameter

   Return:
      number of lines in given file
*/
int tfstextcount(int fd, int *maxlnlen);

/*
   Function: tfstextappendln
   Write line at end of file

   Parameters:
      fd - file descriptor (file must be opened for writing)
      text - line to write

   Return:
      number of readed bytes (or error code)

*/
int tfstextappendln(int fd, char *text);

/*
   Function: tfstextinsertln
   insert line as given number

   Parameters:
      fd - file descriptor (file must be opened for writing)
      num - where line may be placed
      text - line to place

   Return:
      number of written bytes or error code
*/
int tfstextinsertln(int fd, const int linenum, char *text);

/*
   Function: tfstextwriteln
   Write line at current position

   Parameters:
      fd - file descriptor
      text - text to write

   Return:
      number of written bytes
   
   Note:
	  If in writting place of file are some data, it will be overwrited
*/
int tfstextwriteln(int fd, char *text);

/*
   Function: tfstextreplaceln
   Replaces line with given text

   Parameters:
      fd - file descriptor (file must be opened for writing)
      num - line number to replace
      text - line to write

   Return:
      bytes written or error code

*/
int tfstextreplaceln(int fd, const int linenum, char *text);

/*
   Function: tfstextdeleteln
   Delete line

   Parameters:
      fd - file descriptor (file must be opened for writing)
      num - where line may be placed

   Return:
      TFS_OKAY or error code
*/
int tfstextdeleteln(int fd, int linenum);

/*
   Function: tfstextmode
   Return given file EOF mode

   Parameters:
      fd - file descriptor

   Return:
      WIN_MODE, MAC_MODE or LIN_MODE
*/
int tfstextmode(const int fd);

/*
   Function: tfstextwriteeol
   Write EOL-char to the file

   Parameters:
      fd - file descriptor (file must be opened for writing)
      mode - EOL type windows (WIN_MODE), linux (LIN_MODE) or macintosh (MAC_MODE)

   Return:
      number of readed bytes (or error code)

   Note:
      EOL is \r\n (windows), \n(linux), or \r(mac)
      where \n = 0x0A = LF, \r = 0x0D = CR
*/
int tfstextwriteeol(const int fd, const int mode);

/*
   Function: tfstextgotoln
   Move file position to start of given line

   Parameters:
      fd - file descriptor
      num - line number

   Return:
      TFS_OKAY or error code
*/
int tfstextgotoln(int fd, int num);

/*
   Function: tfstextgotonextln
   Move position to start of next line

   Parameters:
      fd - file descriptor

   Return:
      number of bytes from current position to start of next line (or error code)
*/
int tfstextgotonextln(int fd);


#endif /* TFSTEXT_H */
